# -*- coding: utf-8 -*-
# 16/6/28
# create by: snower

from tornado import gen
from peewee import MySQLDatabase as BaseMySQLDatabase, IndexMetadata, ColumnMetadata, ForeignKeyMetadata, sort_models_topologically
from .transaction import Transaction as BaseTransaction, TransactionFuture as BaseTransactionFuture

try:
    import tormysql
except ImportError:
    tormysql = None

class AsyncMySQLDatabase(BaseMySQLDatabase):
    def begin(self):
        raise NotImplementedError

    def commit(self):
        raise NotImplementedError

    def rollback(self):
        raise NotImplementedError

    def push_execution_context(self, transaction):
        raise NotImplementedError

    def pop_execution_context(self):
        raise NotImplementedError

    def execution_context_depth(self):
        raise NotImplementedError

    def execution_context(self, with_transaction=True, transaction_type=None):
        raise NotImplementedError

    def push_transaction(self, transaction):
        raise NotImplementedError

    def pop_transaction(self):
        raise NotImplementedError

    def transaction_depth(self):
        raise NotImplementedError

    @gen.coroutine
    def transaction(self, transaction_type=None):
        raise NotImplementedError

    def commit_on_success(self, func):
        raise NotImplementedError

    def savepoint(self, sid=None):
        raise NotImplementedError

    def atomic(self, transaction_type=None):
        raise NotImplementedError

    @gen.coroutine
    def get_tables(self, schema=None):
        rows = yield self.execute_sql('SHOW TABLES')
        raise gen.Return([row for row, in rows])

    @gen.coroutine
    def get_indexes(self, table, schema=None):
        cursor = yield self.execute_sql('SHOW INDEX FROM `%s`' % table)
        unique = set()
        indexes = {}
        for row in cursor.fetchall():
            if not row[1]:
                unique.add(row[2])
            indexes.setdefault(row[2], [])
            indexes[row[2]].append(row[4])
        raise gen.Return([IndexMetadata(name, None, indexes[name], name in unique, table)
                          for name in indexes])

    @gen.coroutine
    def get_columns(self, table, schema=None):
        sql = """
                SELECT column_name, is_nullable, data_type
                FROM information_schema.columns
                WHERE table_name = %s AND table_schema = DATABASE()"""
        cursor = yield self.execute_sql(sql, (table,))
        pks = set(self.get_primary_keys(table))
        raise gen.Return([ColumnMetadata(name, dt, null == 'YES', name in pks, table)
                          for name, null, dt in cursor.fetchall()])

    @gen.coroutine
    def get_primary_keys(self, table, schema=None):
        cursor = yield self.execute_sql('SHOW INDEX FROM `%s`' % table)
        raise gen.Return([row[4] for row in cursor.fetchall() if row[2] == 'PRIMARY'])

    @gen.coroutine
    def get_foreign_keys(self, table, schema=None):
        query = """
                SELECT column_name, referenced_table_name, referenced_column_name
                FROM information_schema.key_column_usage
                WHERE table_name = %s
                    AND table_schema = DATABASE()
                    AND referenced_table_name IS NOT NULL
                    AND referenced_column_name IS NOT NULL"""
        cursor = yield self.execute_sql(query, (table,))
        raise gen.Return([ForeignKeyMetadata(column, dest_table, dest_column, table)
                             for column, dest_table, dest_column in cursor.fetchall()])

    @gen.coroutine
    def create_table(self, model_class, safe=False):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.create_table(model_class, safe))
        raise gen.Return(cursor)

    @gen.coroutine
    def create_tables(self, models, safe=False):
        for m in sort_models_topologically(models):
            yield m.create_table(fail_silently=safe)

    @gen.coroutine
    def create_index(self, model_class, fields, unique=False):
        qc = self.compiler()
        if not isinstance(fields, (list, tuple)):
            raise ValueError('Fields passed to "create_index" must be a list '
                             'or tuple: "%s"' % fields)
        fobjs = [
            model_class._meta.fields[f] if isinstance(f, basestring) else f
            for f in fields]
        cursor = yield self.execute_sql(*qc.create_index(model_class, fobjs, unique))
        raise gen.Return(cursor)

    @gen.coroutine
    def create_foreign_key(self, model_class, field, constraint=None):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.create_foreign_key(
            model_class, field, constraint))
        raise gen.Return(cursor)

    @gen.coroutine
    def create_sequence(self, seq):
        if self.sequences:
            qc = self.compiler()
            cursor = yield self.execute_sql(*qc.create_sequence(seq))
            raise gen.Return(cursor)

    @gen.coroutine
    def drop_table(self, model_class, fail_silently=False, cascade=False):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.drop_table(
            model_class, fail_silently, cascade))
        raise gen.Return(cursor)

    @gen.coroutine
    def drop_tables(self, models, safe=False, cascade=False):
        for m in reversed(sort_models_topologically(models)):
            yield m.drop_table(fail_silently=safe, cascade=cascade)

    @gen.coroutine
    def truncate_table(self, model_class, restart_identity=False,
                       cascade=False):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.truncate_table(
            model_class, restart_identity, cascade))
        raise gen.Return(cursor)

    @gen.coroutine
    def truncate_tables(self, models, restart_identity=False, cascade=False):
        for model in reversed(sort_models_topologically(models)):
            yield model.truncate_table(restart_identity, cascade)

    @gen.coroutine
    def drop_sequence(self, seq):
        if self.sequences:
            qc = self.compiler()
            cursor = yield self.execute_sql(*qc.drop_sequence(seq))
            raise gen.Return(cursor)


class Transaction(BaseTransaction, AsyncMySQLDatabase):
    def __init__(self, database):
        AsyncMySQLDatabase.__init__(self, database.database)

        self.database = database
        self.connection = None

class TransactionFuture(BaseTransactionFuture):
    def __init__(self, database, args_name):
        super(TransactionFuture, self).__init__(args_name)

        self.transaction = Transaction(database)
        self._future = None


class MySQLDatabase(AsyncMySQLDatabase):
    def __init__(self, *args, **kwargs):
        kwargs["threadlocals"] = False

        super(MySQLDatabase, self).__init__(*args, **kwargs)

        self._closed = True
        self._conn_pool = None

    def _connect(self, database, **kwargs):
        conn_kwargs = {
            'charset': 'utf8',
            'use_unicode': True,
        }
        conn_kwargs.update(kwargs)
        if 'password' in conn_kwargs:
            conn_kwargs['passwd'] = conn_kwargs.pop('password')
        return tormysql.ConnectionPool(db=database, **conn_kwargs)

    def close(self):
        with self._conn_lock:
            if self.deferred:
                raise Exception('Error, database not properly initialized '
                                'before closing connection')
            with self.exception_wrapper:
                if not self._closed and self._conn_pool:
                    self._conn_pool.close()
                    self._closed = True

    @gen.coroutine
    def get_conn(self):
        if self._closed:
            with self.exception_wrapper:
                self._conn_pool = self._connect(self.database, **self.connect_kwargs)
                self._closed = False
                self.initialize_connection(self._conn_pool)
        conn = yield self._conn_pool.Connection()
        raise gen.Return(conn)

    @gen.coroutine
    def execute_sql(self, sql, params=None, require_commit=True):
        with self.exception_wrapper:
            conn = yield self.get_conn()
            try:
                cursor = conn.cursor()
                yield cursor.execute(sql, params or ())
                yield cursor.close()
            except Exception:
                if self.get_autocommit() and self.autorollback:
                    yield conn.rollback()
                raise
            else:
                if require_commit and self.get_autocommit():
                    yield conn.commit()
            finally:
                self._close(conn)
        raise gen.Return(cursor)

    def transaction(self, args_name = "transaction"):
        return TransactionFuture(self, args_name)

    def commit_on_success(self, func):
        return self.transaction()(func)


# -*- coding: utf-8 -*-
# 16/6/29
# create by: snower

from tornado import gen
from peewee import Model as BaseModel, ModelAlias, IntegrityError
from .query import SelectQuery, UpdateQuery, InsertQuery, DeleteQuery, RawQuery
from .transaction import TransactionFuture

class Model(BaseModel):
    @classmethod
    def select(cls, *selection):
        query = SelectQuery(cls, *selection)
        if cls._meta.order_by:
            query = query.order_by(*cls._meta.order_by)
        return query

    @classmethod
    def update(cls, __data=None, **update):
        fdict = __data or {}
        fdict.update([(cls._meta.fields[f], update[f]) for f in update])
        return UpdateQuery(cls, fdict)

    @classmethod
    def insert(cls, __data=None, **insert):
        fdict = __data or {}
        fdict.update([(cls._meta.fields[f], insert[f]) for f in insert])
        return InsertQuery(cls, fdict)

    @classmethod
    def insert_many(cls, rows, validate_fields=True):
        return InsertQuery(cls, rows=rows, validate_fields=True)

    @classmethod
    def insert_from(cls, fields, query):
        return InsertQuery(cls, fields=fields, query=query)

    @classmethod
    def delete(cls):
        return DeleteQuery(cls)

    @classmethod
    def raw(cls, sql, *params):
        return RawQuery(cls, sql, *params)

    @classmethod
    def use(cls, database):
        return Using(cls, database)

    @classmethod
    @gen.coroutine
    def create(cls, **query):
        inst = cls(**query)
        yield inst.save(force_insert=True)
        inst._prepare_instance()
        raise gen.Return(inst)

    @classmethod
    @gen.coroutine
    def get(cls, *query, **kwargs):
        sq = cls.select().naive()
        if query:
            sq = sq.where(*query)
        if kwargs:
            sq = sq.filter(**kwargs)
        result = yield sq.get()
        raise gen.Return(result)

    @classmethod
    @gen.coroutine
    def get_or_create(cls, **kwargs):
        defaults = kwargs.pop('defaults', {})
        query = cls.select()
        for field, value in kwargs.items():
            if '__' in field:
                query = query.filter(**{field: value})
            else:
                query = query.where(getattr(cls, field) == value)

        try:
            result = (yield query.get()), False
        except cls.DoesNotExist:
            try:
                params = dict((k, v) for k, v in kwargs.items()
                              if '__' not in k)
                params.update(defaults)
                result = (yield cls.create(**params)), True
            except IntegrityError as exc:
                try:
                    result = (yield query.get()), False
                except cls.DoesNotExist:
                    raise exc
        raise gen.Return(result)

    @classmethod
    @gen.coroutine
    def create_or_get(cls, **kwargs):
        try:
            result = (yield cls.create(**kwargs)), True
        except IntegrityError:
            query = []  # TODO: multi-column unique constraints.
            for field_name, value in kwargs.items():
                field = getattr(cls, field_name)
                if field.unique or field.primary_key:
                    query.append(field == value)
            result = (yield cls.get(*query)), False
        raise gen.Return(result)

    @classmethod
    @gen.coroutine
    def table_exists(cls):
        kwargs = {}
        if cls._meta.schema:
            kwargs['schema'] = cls._meta.schema
        tables = yield cls._meta.database.get_tables(**kwargs)
        raise gen.Return(cls._meta.db_table in tables)

    @classmethod
    @gen.coroutine
    def create_table(cls, fail_silently=False):
        if fail_silently and (yield cls.table_exists()):
            return

        db = cls._meta.database
        pk = cls._meta.primary_key
        if db.sequences and pk is not False and pk.sequence:
            if not (yield db.sequence_exists(pk.sequence)):
                yield db.create_sequence(pk.sequence)

        yield db.create_table(cls)
        yield cls._create_indexes()

    @classmethod
    @gen.coroutine
    def _create_indexes(cls):
        db = cls._meta.database
        for field in cls._fields_to_index():
            yield db.create_index(cls, [field], field.unique)

        if cls._meta.indexes:
            for fields, unique in cls._meta.indexes:
                yield db.create_index(cls, fields, unique)

    @classmethod
    @gen.coroutine
    def drop_table(cls, fail_silently=False, cascade=False):
        yield cls._meta.database.drop_table(cls, fail_silently, cascade)

    @classmethod
    @gen.coroutine
    def truncate_table(cls, restart_identity=False, cascade=False):
        yield cls._meta.database.truncate_table(cls, restart_identity, cascade)

    @gen.coroutine
    def save(self, force_insert=False, only=None):
        field_dict = dict(self._data)
        if self._meta.primary_key is not False:
            pk_field = self._meta.primary_key
            pk_value = self._get_pk_value()
        else:
            pk_field = pk_value = None
        if only:
            field_dict = self._prune_fields(field_dict, only)
        elif self._meta.only_save_dirty and not force_insert:
            field_dict = self._prune_fields(
                field_dict,
                self.dirty_fields)
            if not field_dict:
                self._dirty.clear()
                raise gen.Return(False)

        self._populate_unsaved_relations(field_dict)
        if pk_value is not None and not force_insert:
            if self._meta.composite_key:
                for pk_part_name in pk_field.field_names:
                    field_dict.pop(pk_part_name, None)
            else:
                field_dict.pop(pk_field.name, None)
            rows = yield self.update(**field_dict).where(self._pk_expr()).execute()
        elif pk_field is None:
            yield self.insert(**field_dict).execute()
            rows = 1
        else:
            pk_from_cursor = yield self.insert(**field_dict).execute()
            if pk_from_cursor is not None:
                pk_value = pk_from_cursor
            self._set_pk_value(pk_value)
            rows = 1
        self._dirty.clear()
        raise gen.Return(rows)

    @gen.coroutine
    def delete_instance(self, recursive=False, delete_nullable=False):
        if recursive:
            dependencies = self.dependencies(delete_nullable)
            for query, fk in reversed(list(dependencies)):
                model = fk.model_class
                if fk.null and not delete_nullable:
                    yield model.update(**{fk.name: None}).where(query).execute()
                else:
                    yield model.delete().where(query).execute()
        result = yield self.delete().where(self._pk_expr()).execute()
        raise gen.Return(result)


class Using(object):
    model_class = None
    database = None

    def __init__(self, model_class, database):
        self.model_class = model_class
        if isinstance(database, TransactionFuture):
            database = database.transaction
        self.database = database

    def __getattr__(self, key):
        attr = getattr(self.model_class, key)
        if not callable(attr):
            return attr

        def attr_proxy(*args, **kwargs):
            ori_database = self.model_class._meta.database
            self.model_class._meta.database = self.database
            try:
                return attr(*args, **kwargs)
            finally:
                self.model_class._meta.database = ori_database
        return attr_proxy

    def __setattr__(self, key, value):
        if self.model_class and self.database:
            return setattr(self.model_class, key, value)
        return super(Using, self).__setattr__(key, value)

    def alias(self):
        model_alias = ModelAlias(self.model_class)
        return Using(model_alias, self.database)


# -*- coding: utf-8 -*-
# 16/6/28
# create by: snower

import re
from tornado import gen
from peewee import PostgresqlDatabase as BasePostgresqlDatabase, IndexMetadata, ColumnMetadata, ForeignKeyMetadata, sort_models_topologically
from .transaction import Transaction as BaseTransaction, TransactionFuture as BaseTransactionFuture

try:
    import momoko
except ImportError:
    momoko = None


def _param_escape(s, re_escape=re.compile(r"([\\'])"), re_space=re.compile(r'\s')):
    if not s: return "''"

    s = re_escape.sub(r'\\\1', s)
    if re_space.search(s):
        s = "'" + s + "'"

    return s

class AsyncPostgresqlDatabase(BasePostgresqlDatabase):
    def begin(self):
        raise NotImplementedError

    def commit(self):
        raise NotImplementedError

    def rollback(self):
        raise NotImplementedError

    def push_execution_context(self, transaction):
        raise NotImplementedError

    def pop_execution_context(self):
        raise NotImplementedError

    def execution_context_depth(self):
        raise NotImplementedError

    def execution_context(self, with_transaction=True, transaction_type=None):
        raise NotImplementedError

    def push_transaction(self, transaction):
        raise NotImplementedError

    def pop_transaction(self):
        raise NotImplementedError

    def transaction_depth(self):
        raise NotImplementedError

    @gen.coroutine
    def transaction(self, transaction_type=None):
        raise NotImplementedError

    def commit_on_success(self, func):
        raise NotImplementedError

    def savepoint(self, sid=None):
        raise NotImplementedError

    def atomic(self, transaction_type=None):
        raise NotImplementedError

    def _get_pk_sequence(self, model):
        meta = model._meta
        if meta.primary_key is not False and meta.primary_key.sequence:
            return meta.primary_key.sequence
        elif meta.auto_increment:
            return '%s_%s_seq' % (meta.db_table, meta.primary_key.db_column)

    def last_insert_id(self, cursor, model):
        sequence = self._get_pk_sequence(model)
        if not sequence:
            return

        meta = model._meta
        if meta.schema:
            schema = '%s.' % meta.schema
        else:
            schema = ''

        cursor.execute("SELECT CURRVAL('%s\"%s\"')" % (schema, sequence))
        result = cursor.fetchone()[0]
        if self.get_autocommit():
            self.commit()
        return result

    @gen.coroutine
    def get_tables(self, schema='public'):
        query = ('SELECT tablename FROM pg_catalog.pg_tables '
                 'WHERE schemaname = %s ORDER BY tablename')
        raise gen.Return([r for r, in (yield self.execute_sql(query, (schema,))).fetchall()])

    @gen.coroutine
    def get_indexes(self, table, schema='public'):
        query = """
            SELECT
                i.relname, idxs.indexdef, idx.indisunique,
                array_to_string(array_agg(cols.attname), ',')
            FROM pg_catalog.pg_class AS t
            INNER JOIN pg_catalog.pg_index AS idx ON t.oid = idx.indrelid
            INNER JOIN pg_catalog.pg_class AS i ON idx.indexrelid = i.oid
            INNER JOIN pg_catalog.pg_indexes AS idxs ON
                (idxs.tablename = t.relname AND idxs.indexname = i.relname)
            LEFT OUTER JOIN pg_catalog.pg_attribute AS cols ON
                (cols.attrelid = t.oid AND cols.attnum = ANY(idx.indkey))
            WHERE t.relname = %s AND t.relkind = %s AND idxs.schemaname = %s
            GROUP BY i.relname, idxs.indexdef, idx.indisunique
            ORDER BY idx.indisunique DESC, i.relname;"""
        cursor = yield self.execute_sql(query, (table, 'r', schema))
        raise gen.Return([IndexMetadata(row[0], row[1], row[3].split(','), row[2], table)
                for row in cursor.fetchall()])

    @gen.coroutine
    def get_columns(self, table, schema='public'):
        query = """
            SELECT column_name, is_nullable, data_type
            FROM information_schema.columns
            WHERE table_name = %s AND table_schema = %s
            ORDER BY ordinal_position"""
        cursor = yield self.execute_sql(query, (table, schema))
        pks = set((yield self.get_primary_keys(table, schema)))
        raise gen.Return([ColumnMetadata(name, dt, null == 'YES', name in pks, table)
                for name, null, dt in cursor.fetchall()])

    @gen.coroutine
    def get_primary_keys(self, table, schema='public'):
        query = """
            SELECT kc.column_name
            FROM information_schema.table_constraints AS tc
            INNER JOIN information_schema.key_column_usage AS kc ON (
                tc.table_name = kc.table_name AND
                tc.table_schema = kc.table_schema AND
                tc.constraint_name = kc.constraint_name)
            WHERE
                tc.constraint_type = %s AND
                tc.table_name = %s AND
                tc.table_schema = %s"""
        cursor = yield self.execute_sql(query, ('PRIMARY KEY', table, schema))
        raise gen.Return([row for row, in cursor.fetchall()])

    @gen.coroutine
    def get_foreign_keys(self, table, schema='public'):
        sql = """
            SELECT
                kcu.column_name, ccu.table_name, ccu.column_name
            FROM information_schema.table_constraints AS tc
            JOIN information_schema.key_column_usage AS kcu
                ON (tc.constraint_name = kcu.constraint_name AND
                    tc.constraint_schema = kcu.constraint_schema)
            JOIN information_schema.constraint_column_usage AS ccu
                ON (ccu.constraint_name = tc.constraint_name AND
                    ccu.constraint_schema = tc.constraint_schema)
            WHERE
                tc.constraint_type = 'FOREIGN KEY' AND
                tc.table_name = %s AND
                tc.table_schema = %s"""
        cursor = yield self.execute_sql(sql, (table, schema))
        raise gen.Return([ForeignKeyMetadata(row[0], row[1], row[2], table)
                for row in cursor.fetchall()])

    @gen.coroutine
    def sequence_exists(self, sequence):
        res = yield self.execute_sql("""
            SELECT COUNT(*) FROM pg_class, pg_namespace
            WHERE relkind='S'
                AND pg_class.relnamespace = pg_namespace.oid
                AND relname=%s""", (sequence,))
        raise gen.Return(bool(res.fetchone()[0]))

    @gen.coroutine
    def set_search_path(self, *search_path):
        path_params = ','.join(['%s'] * len(search_path))
        yield self.execute_sql('SET search_path TO %s' % path_params, search_path)

    @gen.coroutine
    def create_table(self, model_class, safe=False):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.create_table(model_class, safe))
        raise gen.Return(cursor)

    @gen.coroutine
    def create_tables(self, models, safe=False):
        for m in sort_models_topologically(models):
            yield m.create_table(fail_silently=safe)

    @gen.coroutine
    def create_index(self, model_class, fields, unique=False):
        qc = self.compiler()
        if not isinstance(fields, (list, tuple)):
            raise ValueError('Fields passed to "create_index" must be a list '
                             'or tuple: "%s"' % fields)
        fobjs = [
            model_class._meta.fields[f] if isinstance(f, basestring) else f
            for f in fields]
        cursor = yield self.execute_sql(*qc.create_index(model_class, fobjs, unique))
        raise gen.Return(cursor)

    @gen.coroutine
    def create_foreign_key(self, model_class, field, constraint=None):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.create_foreign_key(
            model_class, field, constraint))
        raise gen.Return(cursor)

    @gen.coroutine
    def create_sequence(self, seq):
        if self.sequences:
            qc = self.compiler()
            cursor = yield self.execute_sql(*qc.create_sequence(seq))
            raise gen.Return(cursor)

    @gen.coroutine
    def drop_table(self, model_class, fail_silently=False, cascade=False):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.drop_table(
            model_class, fail_silently, cascade))
        raise gen.Return(cursor)

    @gen.coroutine
    def drop_tables(self, models, safe=False, cascade=False):
        for m in reversed(sort_models_topologically(models)):
            yield m.drop_table(fail_silently=safe, cascade=cascade)

    @gen.coroutine
    def truncate_table(self, model_class, restart_identity=False,
                       cascade=False):
        qc = self.compiler()
        cursor = yield self.execute_sql(*qc.truncate_table(
            model_class, restart_identity, cascade))
        raise gen.Return(cursor)

    @gen.coroutine
    def truncate_tables(self, models, restart_identity=False, cascade=False):
        for model in reversed(sort_models_topologically(models)):
            yield model.truncate_table(restart_identity, cascade)

    @gen.coroutine
    def drop_sequence(self, seq):
        if self.sequences:
            qc = self.compiler()
            cursor = yield self.execute_sql(*qc.drop_sequence(seq))
            raise gen.Return(cursor)


class Transaction(BaseTransaction, AsyncPostgresqlDatabase):
    def __init__(self, database):
        AsyncPostgresqlDatabase.__init__(self, database.database)

        self.database = database
        self.connection = None

    @gen.coroutine
    def get_cursor(self):
        raise NotImplementedError()

    @gen.coroutine
    def execute_sql(self, sql, params=None, require_commit=True):
        if self.connection is None:
            yield self.begin()

        with self.database.exception_wrapper:
            cursor = yield self.connection.execute(sql, params or ())
        raise gen.Return(cursor)

    @gen.coroutine
    def begin(self):
        self.connection = yield self.database.get_conn()
        yield self.connection.execute("BEGIN;")

    @gen.coroutine
    def commit(self):
        if self.connection:
            yield self.connection.execute("COMMIT;")
            self.close()

    @gen.coroutine
    def rollback(self):
        if self.connection:
            yield self.connection.execute("ROLLBACK;")
            self.close()

class TransactionFuture(BaseTransactionFuture):
    def __init__(self, database, args_name):
        super(TransactionFuture, self).__init__(args_name)

        self.transaction = Transaction(database)
        self._future = None

class PostgresqlDatabase(AsyncPostgresqlDatabase):
    def __init__(self, *args, **kwargs):
        kwargs["threadlocals"] = False

        super(PostgresqlDatabase, self).__init__(*args, **kwargs)

        self._closed = True
        self._conn_pool = None

    def _connect(self, database, **kwargs):
        pool_kwargs = {}
        for key in ['connection_factory', 'cursor_factory', 'size', 'max_size', 'ioloop', 'raise_connect_errors',
                    'reconnect_interval','setsession', 'auto_shrink', 'shrink_delay', 'shrink_period']:
            if key in kwargs:
                pool_kwargs[key] = kwargs.pop(key)

        conn_kwargs = {"dbname": database}
        conn_kwargs.update(kwargs)
        if 'password' in conn_kwargs:
            conn_kwargs['passwd'] = conn_kwargs.pop('password')

        if "database" in conn_kwargs:
            conn_kwargs["dbname"] = conn_kwargs.pop("database")
        if "db" in conn_kwargs:
            conn_kwargs["dbname"] = conn_kwargs.pop("db")
        if "passwd" in conn_kwargs:
            conn_kwargs["password"] = conn_kwargs.pop("passwd")
        dsn = " ".join(["%s=%s" % (k, _param_escape(str(v)))
                                for (k, v) in conn_kwargs.iteritems()])

        if "max_size" not in pool_kwargs:
            pool_kwargs["max_size"] = 32
        return momoko.Pool(dsn = dsn, **pool_kwargs)

    def close(self):
        with self._conn_lock:
            if self.deferred:
                raise Exception('Error, database not properly initialized '
                                'before closing connection')
            with self.exception_wrapper:
                if not self._closed and self._conn_pool:
                    self._conn_pool.close()
                    self._closed = True

    @gen.coroutine
    def get_conn(self):
        if self._closed:
            with self.exception_wrapper:
                self._conn_pool = self._connect(self.database, **self.connect_kwargs)
                self._closed = False
                self.initialize_connection(self._conn_pool)
            yield self._conn_pool.connect()
        conn = yield self._conn_pool.getconn(False)
        raise gen.Return(conn)

    @gen.coroutine
    def execute_sql(self, sql, params=None, require_commit=True):
        with self.exception_wrapper:
            conn = yield self.get_conn()
            try:
                cursor = yield conn.execute(sql, params or ())
            except Exception:
                if self.get_autocommit() and self.autorollback:
                    yield conn.execute("ROLLBACK;")
                raise
            else:
                if require_commit and self.get_autocommit():
                    yield conn.execute("COMMIT;")
            finally:
                self._close(conn)
        raise gen.Return(cursor)

    def transaction(self, args_name = "transaction"):
        return TransactionFuture(self, args_name)

    def commit_on_success(self, func):
        return self.transaction()(func)

    def _close(self, conn):
        self._conn_pool.putconn(conn)


# -*- coding: utf-8 -*-
# 16/8/30
# create by: snower

import sys
from functools import wraps
from tornado.util import raise_exc_info
from tornado import gen

class Transaction(object):
    def __init__(self):
        self.database = None

    def _connect(self, database, **kwargs):
        raise NotImplementedError

    @gen.coroutine
    def get_conn(self):
        if self.connection is None:
            yield self.begin()

        raise gen.Return(self.connection)

    @gen.coroutine
    def get_cursor(self):
        if self.connection is None:
            yield self.begin()

        cursor = yield self.connection.cursor()
        raise gen.Return(cursor)

    @gen.coroutine
    def execute_sql(self, sql, params=None, require_commit=True):
        if self.connection is None:
            yield self.begin()

        with self.database.exception_wrapper:
            cursor = self.connection.cursor()
            yield cursor.execute(sql, params or ())
            yield cursor.close()
        raise gen.Return(cursor)

    @gen.coroutine
    def begin(self):
        self.connection = yield self.database.get_conn()
        yield self.connection.begin()

    @gen.coroutine
    def commit(self):
        if self.connection:
            yield self.connection.commit()
            self.close()

    @gen.coroutine
    def rollback(self):
        if self.connection:
            yield self.connection.rollback()
            self.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.connection is None:
            return

        if exc_type:
            self.rollback()
        else:
            try:
                self.commit()
            except:
                exc_info = sys.exc_info()
                self.rollback()
                raise_exc_info(exc_info)

    def close(self):
        if self.connection:
            self.database._close(self.connection)
            self.connection = None

    def __del__(self):
        self.rollback()

class TransactionFuture(gen.Future):
    def __init__(self, args_name):
        super(TransactionFuture, self).__init__()

        self.args_name = args_name
        self.transaction = None
        self._transaction_begin_future = None

    @gen.coroutine
    def get_conn(self):
        conn = yield self.transaction.get_conn()
        raise gen.Return(conn)

    @gen.coroutine
    def get_cursor(self):
        cursor = yield self.transaction.get_cursor()
        raise gen.Return(cursor)

    @gen.coroutine
    def execute_sql(self, sql, params=None, require_commit=True):
        cursor = yield self.transaction.execute_sql(sql, params, require_commit)
        raise gen.Return(cursor)

    @gen.coroutine
    def begin(self):
        yield self.transaction.begin()

    @gen.coroutine
    def commit(self):
        yield self.transaction.commit()

    @gen.coroutine
    def rollback(self):
        yield self.transaction.rollback()

    def close(self):
        self.transaction.close()

    def __enter__(self):
        return self.transaction.__enter__()

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.transaction.__exit__(exc_type, exc_val, exc_tb)

    def __call__(self, func):
        @gen.coroutine
        @wraps(func)
        def _(*args, **kwargs):
            yield self.transaction.begin()
            try:
                kwargs[self.args_name] = self.transaction
                result = yield func(*args, **kwargs)
            except:
                exc_info = sys.exc_info()
                yield self.transaction.rollback()
                raise_exc_info(exc_info)
            else:
                yield self.transaction.commit()
                raise gen.Return(result)
        return _

    def add_done_callback(self, fn):
        if self._transaction_begin_future is not None:
            return super(TransactionFuture, self).add_done_callback(fn)

        self._transaction_begin_future = self.transaction.begin()

        def on_done(future):
            if future._exc_info is not None:
                self.set_exc_info(future.exc_info())
            else:
                self.set_result(self)

        self._transaction_begin_future.add_done_callback(on_done)
        super(TransactionFuture, self).add_done_callback(fn)


# -*- coding: utf-8 -*-
# 16/6/28
# create by: snower

from setuptools import setup


setup(
    name='torpeewee',
    version='0.0.5',
    packages=['torpeewee'],
    install_requires=[
        'tornado>=4.1',
        'peewee==2.9.1'
    ],
    author=['snower'],
    author_email=['sujian199@gmail.com'],
    url='https://github.com/snower/torpeewee',
    license='MIT',
    keywords=[
        "tornado", "mysql", "postgresql", "orm", "tormysql", "momoko"
    ],
    description='Tornado asynchronous ORM by peewee',
    long_description=open("README.rst").read(),
    zip_safe=False,
)



# -*- coding: utf-8 -*-
# 16/6/28
# create by: snower

from tornado import gen
from peewee import SQL, operator, RESULTS_TUPLES, RESULTS_DICTS, RESULTS_NAIVE
from peewee import SelectQuery as BaseSelectQuery, UpdateQuery as BaseUpdateQuery, InsertQuery as BaseInsertQuery, DeleteQuery as BaseDeleteQuery, RawQuery as BaseRawQuery


class QueryIsDoneError(Exception):
    pass


class SelectQuery(gen.Future, BaseSelectQuery):
    def __init__(self, *args, **kwargs):
        BaseSelectQuery.__init__(self, *args, **kwargs)
        gen.Future.__init__(self)

        self._future = None

    @gen.coroutine
    def _execute(self):
        sql, params = self.sql()
        result = yield self.database.execute_sql(sql, params, self.require_commit)
        raise gen.Return(result)

    @gen.coroutine
    def scalar(self, as_tuple=False, convert=False):
        if convert:
            row = yield self.tuples().first()
        else:
            row = (yield self._execute()).fetchone()
        if row and not as_tuple:
            raise gen.Return(row[0])
        else:
            raise gen.Return(row)

    @gen.coroutine
    def execute(self):
        if self._dirty or self._qr is None:
            model_class = self.model_class
            query_meta = self.get_query_meta()
            ResultWrapper = self._get_result_wrapper()
            self._qr = ResultWrapper(model_class, (yield self._execute()), query_meta)
            self._dirty = False
            raise gen.Return(self._qr)
        else:
            raise gen.Return(self._qr)

    @gen.coroutine
    def get(self):
        clone = self.paginate(1, 1)
        try:
            result = next((yield clone.execute()))
        except StopIteration:
            raise self.model_class.DoesNotExist(
                'Instance matching query does not exist:\nSQL: %s\nPARAMS: %s'
                % self.sql())
        raise gen.Return(result)

    @gen.coroutine
    def first(self):
        res = yield self.execute()
        res.fill_cache(1)
        try:
            result = res._result_cache[0]
        except IndexError:
            result = None
        raise gen.Return(result)

    @gen.coroutine
    def exists(self):
        clone = self.paginate(1, 1)
        clone._select = [SQL('1')]
        raise gen.Return(bool((yield clone.scalar())))

    def __iter__(self):
        raise NotImplementedError()

    @gen.coroutine
    def iterator(self):
        raise gen.Return(iter((yield self.execute()).iterator()))

    @gen.coroutine
    def __getitem__(self, value):
        res = yield self.execute()
        if isinstance(value, slice):
            index = value.stop
        else:
            index = value
        if index is not None and index >= 0:
            index += 1
        res.fill_cache(index)
        raise gen.Return(res._result_cache[value])

    def __len__(self):
        raise NotImplementedError()

    @gen.coroutine
    def len(self):
        raise gen.Return(len((yield self.execute())))

    def add_done_callback(self, fn):
        if self._future is not None:
            raise QueryIsDoneError()

        self._future = self.execute()

        def on_done(future):
            if future._exc_info is not None:
                self.set_exc_info(future.exc_info())
            else:
                self.set_result(future.result())

        self._future.add_done_callback(on_done)
        super(SelectQuery, self).add_done_callback(fn)


class UpdateQuery(gen.Future, BaseUpdateQuery):
    def __init__(self, *args, **kwargs):
        BaseUpdateQuery.__init__(self, *args, **kwargs)
        gen.Future.__init__(self)

        self._future = None

    @gen.coroutine
    def _execute(self):
        sql, params = self.sql()
        result = yield self.database.execute_sql(sql, params, self.require_commit)
        raise gen.Return(result)

    @gen.coroutine
    def scalar(self, as_tuple=False, convert=False):
        if convert:
            row = yield self.tuples().first()
        else:
            row = (yield self._execute()).fetchone()
        if row and not as_tuple:
            raise gen.Return(row[0])
        else:
            raise gen.Return(row)

    @gen.coroutine
    def _execute_with_result_wrapper(self):
        ResultWrapper = self.get_result_wrapper()
        meta = (self._returning, {self.model_class: []})
        self._qr = ResultWrapper(self.model_class, (yield self._execute()), meta)
        raise gen.Return(self._qr)

    @gen.coroutine
    def execute(self):
        if self._returning is not None and self._qr is None:
            result = yield self._execute_with_result_wrapper()
        elif self._qr is not None:
            result = self._qr
        else:
            result = self.database.rows_affected((yield self._execute()))
        raise gen.Return(result)

    def __iter__(self):
        if not self.model_class._meta.database.returning_clause:
            raise ValueError('UPDATE queries cannot be iterated over unless '
                             'they specify a RETURNING clause, which is not '
                             'supported by your database.')
        raise NotImplementedError()

    def iterator(self):
        raise gen.Return(iter((yield self.execute()).iterator()))

    def add_done_callback(self, fn):
        if self._future is not None:
            raise QueryIsDoneError()

        self._future = self.execute()

        def on_done(future):
            if future._exc_info is not None:
                self.set_exc_info(future.exc_info())
            else:
                self.set_result(future.result())

        self._future.add_done_callback(on_done)
        super(UpdateQuery, self).add_done_callback(fn)


class InsertQuery(gen.Future, BaseInsertQuery):
    def __init__(self, *args, **kwargs):
        BaseInsertQuery.__init__(self, *args, **kwargs)
        gen.Future.__init__(self)

        self._future = None

    @gen.coroutine
    def _execute(self):
        sql, params = self.sql()
        result = yield self.database.execute_sql(sql, params, self.require_commit)
        raise gen.Return(result)

    @gen.coroutine
    def scalar(self, as_tuple=False, convert=False):
        if convert:
            row = yield self.tuples().first()
        else:
            row = (yield self._execute()).fetchone()
        if row and not as_tuple:
            raise gen.Return(row[0])
        else:
            raise gen.Return(row)

    @gen.coroutine
    def _execute_with_result_wrapper(self):
        ResultWrapper = self.get_result_wrapper()
        meta = (self._returning, {self.model_class: []})
        self._qr = ResultWrapper(self.model_class, (yield self._execute()), meta)
        raise gen.Return(self._qr)

    @gen.coroutine
    def _insert_with_loop(self):
        id_list = []
        last_id = None
        return_id_list = self._return_id_list
        for row in self._rows:
            last_id = (yield InsertQuery(self.model_class, row)
                       .upsert(self._upsert)
                       .execute())
            if return_id_list:
                id_list.append(last_id)

        if return_id_list:
            raise gen.Return(id_list)
        else:
            raise gen.Return(last_id)

    @gen.coroutine
    def execute(self):
        insert_with_loop = (
            self._is_multi_row_insert and
            self._query is None and
            self._returning is None and
            not self.database.insert_many)
        if insert_with_loop:
            result = yield self._insert_with_loop()
            raise gen.Return(result)

        if self._returning is not None and self._qr is None:
            result =  yield self._execute_with_result_wrapper()
            raise gen.Return(result)
        elif self._qr is not None:
            raise gen.Return(self._qr)
        else:
            cursor = yield self._execute()
            if not self._is_multi_row_insert:
                if self.database.insert_returning:
                    pk_row = cursor.fetchone()
                    meta = self.model_class._meta
                    clean_data = [
                        field.python_value(column)
                        for field, column
                        in zip(meta.get_primary_key_fields(), pk_row)]
                    if self.model_class._meta.composite_key:
                        raise gen.Return(clean_data)
                    raise gen.Return(clean_data[0])
                raise gen.Return(self.database.last_insert_id(cursor, self.model_class))
            elif self._return_id_list:
                raise gen.Return(map(operator.itemgetter(0), cursor.fetchall()))
            else:
                raise gen.Return(True)

    def add_done_callback(self, fn):
        if self._future is not None:
            raise QueryIsDoneError()

        self._future = self.execute()

        def on_done(future):
            if future._exc_info is not None:
                self.set_exc_info(future.exc_info())
            else:
                self.set_result(future.result())

        self._future.add_done_callback(on_done)
        super(InsertQuery, self).add_done_callback(fn)


class DeleteQuery(gen.Future, BaseDeleteQuery):
    def __init__(self, *args, **kwargs):
        BaseDeleteQuery.__init__(self, *args, **kwargs)
        gen.Future.__init__(self)

        self._future = None

    @gen.coroutine
    def _execute(self):
        sql, params = self.sql()
        result = yield self.database.execute_sql(sql, params, self.require_commit)
        raise gen.Return(result)

    @gen.coroutine
    def scalar(self, as_tuple=False, convert=False):
        if convert:
            row = yield self.tuples().first()
        else:
            row = (yield self._execute()).fetchone()
        if row and not as_tuple:
            raise gen.Return(row[0])
        else:
            raise gen.Return(row)

    @gen.coroutine
    def _execute_with_result_wrapper(self):
        ResultWrapper = self.get_result_wrapper()
        meta = (self._returning, {self.model_class: []})
        self._qr = ResultWrapper(self.model_class, (yield self._execute()), meta)
        raise gen.Return(self._qr)

    @gen.coroutine
    def execute(self):
        if self._returning is not None and self._qr is None:
            result = yield self._execute_with_result_wrapper()
        elif self._qr is not None:
            result = self._qr
        else:
            result = self.database.rows_affected((yield self._execute()))
        raise gen.Return(result)

    def add_done_callback(self, fn):
        if self._future is not None:
            raise QueryIsDoneError()

        self._future = self.execute()

        def on_done(future):
            if future._exc_info is not None:
                self.set_exc_info(future.exc_info())
            else:
                self.set_result(future.result())

        self._future.add_done_callback(on_done)
        super(DeleteQuery, self).add_done_callback(fn)


class RawQuery(gen.Future, BaseRawQuery):
    def __init__(self, *args, **kwargs):
        BaseRawQuery.__init__(self, *args, **kwargs)
        gen.Future.__init__(self)

        self._future = None

    @gen.coroutine
    def _execute(self):
        sql, params = self.sql()
        result = yield self.database.execute_sql(sql, params, self.require_commit)
        raise gen.Return(result)

    @gen.coroutine
    def scalar(self, as_tuple=False, convert=False):
        if convert:
            row = yield self.tuples().first()
        else:
            row = (yield self._execute()).fetchone()
        if row and not as_tuple:
            raise gen.Return(row[0])
        else:
            raise gen.Return(row)

    def execute(self):
        if self._qr is None:
            if self._tuples:
                QRW = self.database.get_result_wrapper(RESULTS_TUPLES)
            elif self._dicts:
                QRW = self.database.get_result_wrapper(RESULTS_DICTS)
            else:
                QRW = self.database.get_result_wrapper(RESULTS_NAIVE)
            self._qr = QRW(self.model_class, (yield self._execute()), None)
        raise gen.Return(self._qr)

    def add_done_callback(self, fn):
        if self._future is not None:
            raise QueryIsDoneError()

        self._future = self.execute()

        def on_done(future):
            if future._exc_info is not None:
                self.set_exc_info(future.exc_info())
            else:
                self.set_result(future.result())

        self._future.add_done_callback(on_done)
        super(RawQuery, self).add_done_callback(fn)

    def __iter__(self):
        raise NotImplementedError()

    def iterator(self):
        raise gen.Return(iter((yield self.execute())))